package services.as400Connector.server;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

import libraries.ejbUtils.logging.Logger;

import com.ibm.as400.access.AS400;
import com.ibm.as400.access.AS400Message;
import com.ibm.as400.data.PcmlException;
import com.ibm.as400.data.ProgramCallDocument;

public class AS400ManagedConnection implements ManagedConnection {

  private List<ConnectionEventListener> listeners;
  private List<AS400ConnectionImpl>     allCxHandles;
  private AS400ManagedConnectionFactory mcf;
  private PrintWriter                   out;
  private AS400                         as400System;
  private boolean                       destroyed;
  private AS400XAResource               as400XAResource;
  private AS400LocalTransaction         as400LocalTransaction;
  private boolean                       isInTransaction;
  private Logger                        log;
  private long                          connectionStartTime;
  
  public AS400ManagedConnection(AS400ManagedConnectionFactory mcf, AS400 as400System) {
    this.destroyed           = false;
    this.mcf                 = mcf;
    this.as400System         = as400System;
    this.listeners           = new ArrayList<ConnectionEventListener>();
    this.allCxHandles        = new ArrayList<AS400ConnectionImpl>();
    this.isInTransaction     = false;
    this.log                 = mcf.getLogger();
    this.connectionStartTime = (new Date()).getTime();
  }

  /**
   * See association stuff - Spec 7.11
   */
  public void associateConnection(Object connection) throws ResourceException {
    log.info("Associating a new connection");
    checkIfDestroyed();
    ((AS400ConnectionImpl) connection).associateConnection(this);
  }

  /**
   * Reset client specific state, invalidate all handles, prepare the connection
   * to be put back in the pool Should not close the physical connection Spec 6.5.4.4
   */
  public void cleanup() throws ResourceException {
    log.info("Cleaning up the managed connection");
    checkIfDestroyed();
    clearAllHandles();
    long now = (new Date()).getTime();
    if ( (now - connectionStartTime) > (mcf.getMaxConnectionLifetimeMinutes() * 60 * 1000) ) {
      refreshConnection();
      connectionStartTime = 0;
    }
  }

  private void clearAllHandles() {
    for (AS400ConnectionImpl cx : allCxHandles) {
      cx.invalidate();
    }
    allCxHandles.clear();
  }

  /**
   * we need to do this, because the as400 keeps a bunch of temp resources around that can get out of hand if the connection is kept
   * open for extended periods. We do not create a new AS400 obj. Once we've disconnected, the next call to the obj will reconnect automatically (see jtopen docs p434)
   */
  private void refreshConnection() {
    as400System.disconnectAllServices();
  }

  /**
   * close the physical connection
   */
  public void destroy() throws ResourceException {
    log.info("Going to destroy the managed connection");
    if (destroyed) {
      return;
    }
    clearAllHandles();
    as400System.disconnectAllServices();
    destroyed = true;
  }

  /**
   * create a handle that the ultimate client can use (application level
   * connection handle)
   */
  public Object getConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
    log.info("Returning a client handle to the ManagedConnection");
    checkIfDestroyed();
    AS400ConnectionImpl cxHandle = new AS400ConnectionImpl(this);
    addCxHandle(cxHandle);
    return cxHandle;
  }

  public LocalTransaction getLocalTransaction() throws ResourceException {
    log.debug("Returning a reference to the local transaction");
    checkIfDestroyed();
    synchronized (this) {
      if (as400LocalTransaction == null) {
        as400LocalTransaction = new AS400LocalTransaction(this);
      }
    }
    return as400LocalTransaction;
  }

  void commitTransaction() throws ResourceException {
    log.info("Going to commit the transaction");
    try {
      ProgramCallDocument pcml = new ProgramCallDocument(as400System, "transactions");
      pcml.setValue("DS.requestName", "COMMIT");
      call(pcml);
    } catch (PcmlException e) {
      log.error("An error occured while trying to commit the transaction",e);
      notifyConnectionError();
      throw new ResourceException("Failed to commit AS400 transaction", e);
    } finally {
      // whatever happens, the transaction ends
      isInTransaction = false;
    }
  }

  void rollbackTransaction() throws ResourceException {
    log.info("Going to rollback the transaction");
    try {
      ProgramCallDocument pcml = new ProgramCallDocument(as400System, "transactions");
      pcml.setValue("DS.requestName", "ROLLBACK");
      call(pcml);
    } catch (PcmlException e) {
      log.error("An error occured while trying to rollback the transaction",e);
      notifyConnectionError();
      throw new ResourceException("Failed to rollback AS400 transaction", e);
    } finally {
      // whatever happens, the transaction ends
      isInTransaction = false;
    }
  }

  private void call(ProgramCallDocument pcml) throws ResourceException {

    String progName = "DS";

    try {
      pcml.setValue("DS.DataBase", getDatabase());

      // Request to call the API- User will be prompted to sign on to the system
      boolean rc = pcml.callProgram(progName);

      // If return code is false, we received messages from the server
      if (!rc) {
        String returnCode = (String) pcml.getValue("DS.returnCode");
        String returnMessage = (String) pcml.getValue("DS.returnMessage");
        String requestName = (String) pcml.getValue("DS.requestName");

        StringBuffer message = new StringBuffer();
        message.append("Call to QSYRUSRI failed : Return code = " + returnCode + ", requestName = " + requestName + ", message = " + returnMessage + "\n");
        message.append("AS400 messages : \n");

        // Retrieve list of server messages
        AS400Message[] msgs = pcml.getMessageList(progName);
        for (int m = 0; msgs != null && m < msgs.length; m++) {
          message.append(msgs[m].getID() + " - " + msgs[m].getText() + "\n");
        }

        throw new ResourceException(message.toString());
      }
    } catch (PcmlException e) {
      log.error("Failed to commit or rollback AS400 transaction", e);
      throw new ResourceException("Failed to commit or rollback AS400 transaction", e);
    }
  }

  public XAResource getXAResource() throws ResourceException {
    // throw new NotSupportedException("XA trans not supported yet");
    log.debug("Returning the XAResource");
    checkIfDestroyed();
    synchronized (this) {
      if (as400XAResource == null) {
        as400XAResource = new AS400XAResource(this);
      }
    }
    return as400XAResource;
  }

  public PrintWriter getLogWriter() throws ResourceException {
    return out;
  }

  public void setLogWriter(PrintWriter out) throws ResourceException {
    this.out = out;
  }

  public ManagedConnectionMetaData getMetaData() throws ResourceException {
    checkIfDestroyed();
    return new AS400ManagedConnectionMetaData(this);
  }

  public void addConnectionEventListener(ConnectionEventListener listener) {
    listeners.add(listener);
  }

  public void removeConnectionEventListener(ConnectionEventListener listener) {
    listeners.remove(listener);
  }

  void addCxHandle(AS400ConnectionImpl cxHandle) {
    allCxHandles.add(cxHandle);
  }

  void removeCxHandle(AS400ConnectionImpl cxHandle) {
    allCxHandles.remove(cxHandle);
  }

  AS400 getAS400System() {
    checkIfDestroyed();
    if (connectionStartTime == 0) {
      connectionStartTime = (new Date()).getTime();
    }
    return as400System;
  }

  /**
   * @param cxHandle
   *          the handle that is being closed
   */
  public void closeCxHandle(AS400ConnectionImpl cxHandle) {
    log.debug("Going to close the connection handle");
    ConnectionEvent event = new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
    event.setConnectionHandle(cxHandle);

    for (ConnectionEventListener listener : listeners) {
      listener.connectionClosed(event);
    }

  }

  void checkIfDestroyed() {
    if (destroyed) {
      throw new RuntimeException("This ManagedConnection has been destroyed");
    }
  }

  public String getUsername() {
    return mcf.getUserName();
  }

  public String getDatabase() {
    return mcf.getDatabase();
  }

  boolean isInTransaction() {
    return isInTransaction;
  }

  void setInTransaction(boolean isInTransaction) {
    log.info("Setting isInTransaction to " + isInTransaction);
    this.isInTransaction = isInTransaction;
  }

  void notifyConnectionError() {
    log.info("Going to notify listeners of error");
    ConnectionEvent event = new ConnectionEvent(this, ConnectionEvent.CONNECTION_ERROR_OCCURRED);

    for (ConnectionEventListener listener : listeners) {
      listener.connectionErrorOccurred(event);
    }
  }

  public Logger getLogger() {
    return log;
  }
  
}
