package services.as400Connector.server;

/*TODO
 * detect invalid connections 6.5.3.4
 * connection error notification
 */

import java.io.PrintWriter;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.security.auth.Subject;

import libraries.ejbUtils.logging.Logger;
import libraries.ejbUtils.logging.LoggingService;

import com.ibm.as400.access.AS400;

/**
 * This class must be a javabean - spec 17.4.2
 * See the jtopen pdf p434 on how connections are managed
 * basically, the first time the an AS400 obj is used, it will connect. Now it's up to the java program to close the connection. This is done 
 * in AS400ManagedConnection based on the maxConnectionLifetimeMinutes property.
 * @author eduardp
 *
 */
public class AS400ManagedConnectionFactory implements ManagedConnectionFactory {

  private static final long serialVersionUID = 1L;
  
  private PrintWriter printWriter;
  private AS400ConnectionManager defaultCxManager;
  
  //values for these properties are specified on the appserver admin interface 
  //or in the deployment descriptor  and injected by the container using the setters
  private String serverName;
  private String userName;
  private String password;
  private String database;

  private String logPattern;
  private Boolean logUseRolling;
  private String logFileName;
  private String logLevel;
  private Integer logMaxIndex;
  private String logMaxFileSize;
  private Integer maxConnectionLifetimeMinutes;
  
  //The log4j logger we use is not serializable, so we have to mark this as transient, because this is the part of the 
  //connector that is serailized to the jndi 
  private transient Logger logger;
  
  
  public AS400ManagedConnectionFactory() {
    defaultCxManager = new AS400ConnectionManager();
  }
  
  // ---------------------------------------------------------------------------------------------------- [ ConnectionFactory stuff ]
  /**
   * if this method is called, we need to provide a default ConnectionManager, but it will only be called if the connector is used from without the container, which is a scenario that we do not cater for
   */
  public Object createConnectionFactory() throws ResourceException {
    return new AS400ConnectionFactoryImpl(this,defaultCxManager);
  }

  /**
   * The container calls this method if it wants to hand a new factory obj to a client
   */
  public Object createConnectionFactory(ConnectionManager cxManager) throws ResourceException {
    getLogger().info("Creating new connection factory");
    return new AS400ConnectionFactoryImpl(this,cxManager);
  }
  //----------------------------------------------------------------------------------------------------------------------------------- 

  // ---------------------------------------------------------------------------------------------------- [ ManagedConnection stuff ]
  public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
    Logger log = getLogger();
    log.info("Creating new ManagedConnection for server=" + serverName +",username=" + userName);
    AS400 as400System = null; //new AS400(serverName,userName,password);


    AS400ManagedConnection result = new AS400ManagedConnection(this,as400System);
    result.setLogWriter(this.getLogWriter());
    
    return result;
  }

  public ManagedConnection matchManagedConnections(Set connectionSet, Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
    //if I cannot find a suitable connection in the connectionSet, I should return null, then the appserver will create a new ManagedConnection
    Logger log = getLogger();
    if ( (connectionSet == null) || (connectionSet.size() == 0) ) {
      log.debug("Could not find a suitable connection in the connection set, return null so the appserver will create a new connection");
      return null;
    }
    else {
      //all the connections are the same, so return the first one
      log.debug("Returning the first mactched connection");
      Iterator<ManagedConnection> itr = connectionSet.iterator();
      return itr.next();
    }
  }
  //----------------------------------------------------------------------------------------------------------------------------------- 
  
  /**
   * This method is part of the spec. In the sun appserver howerver, printWriter is always null so we can't use it to log.
   * Anyway, we'd rather use our own logger through getLogger() 
   */
  public PrintWriter getLogWriter() throws ResourceException {
    return printWriter;
  }


  public void setLogWriter(PrintWriter out) throws ResourceException {
    this.printWriter = out;
  }


  public String getServerName() {
    return serverName;
  }

  public void setServerName(String serverName) {
    this.serverName = serverName;
  }

  public String getUserName() {
    return userName;
  }

  public void setUserName(String userName) {
    this.userName = userName;
  }

  public String getPassword() {
    return password;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public String getDatabase() {
    return database;
  }

  public void setDatabase(String database) {
    this.database = database;
  }

  /**
   * I might be able to improve the performance of the method. The reference to the log should really be cache, but then serialization fails 
   * even when the cached reference is marked transient
   * @return
   */
  public Logger getLogger() {
    if (logger == null) {
    Properties logProps = new Properties();
    logProps.put(ModuleInfo.MODULE_NAME + ".log.pattern", logPattern);
    logProps.put(ModuleInfo.MODULE_NAME + ".log.useRollingLog",logUseRolling.toString() );
    logProps.put(ModuleInfo.MODULE_NAME + ".log.filename",logFileName );
    logProps.put(ModuleInfo.MODULE_NAME + ".log.level", logLevel);
    logProps.put(ModuleInfo.MODULE_NAME + ".log.maxIndex", logMaxIndex.toString());
    logProps.put(ModuleInfo.MODULE_NAME + ".log.maxFileSize", logMaxFileSize);
    //return LoggingService.getLogger(ModuleInfo.MODULE_NAME, ModuleInfo.MODULE_NAME, logProps);
    logger = LoggingService.getLogger(ModuleInfo.MODULE_NAME, ModuleInfo.MODULE_NAME, logProps);
    }
    return logger;
  }
  
  /**
   * equals and hascode should be based on a complete set of configuration properties that make a ManagedConnectionFactory instance unique ans specific to an EIS (enterprise information system) instance - spec 6.5.3.2
   * In this case the EIS instance is orbit and the servername,username and database name makes the connection unique.
   */
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((database == null) ? 0 : database.hashCode());
    result = prime * result + ((serverName == null) ? 0 : serverName.hashCode());
    result = prime * result + ((userName == null) ? 0 : userName.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    final AS400ManagedConnectionFactory other = (AS400ManagedConnectionFactory) obj;
    if (database == null) {
      if (other.database != null)
        return false;
    } else if (!database.equals(other.database))
      return false;
    if (serverName == null) {
      if (other.serverName != null)
        return false;
    } else if (!serverName.equals(other.serverName))
      return false;
    if (userName == null) {
      if (other.userName != null)
        return false;
    } else if (!userName.equals(other.userName))
      return false;
    return true;
  }

  public String getLogPattern() {
    return logPattern;
  }

  public void setLogPattern(String logPattern) {
    this.logPattern = logPattern;
  }

  public Boolean getLogUseRolling() {
    return logUseRolling;
  }

  public void setLogUseRolling(Boolean logUseRolling) {
    this.logUseRolling = logUseRolling;
  }

  public String getLogFileName() {
    return logFileName;
  }

  public void setLogFileName(String logFileName) {
    this.logFileName = logFileName;
  }

  public String getLogLevel() {
    return logLevel;
  }

  public void setLogLevel(String logLevel) {
    this.logLevel = logLevel;
  }

  public Integer getLogMaxIndex() {
    return logMaxIndex;
  }

  public void setLogMaxIndex(Integer logMaxIndex) {
    this.logMaxIndex = logMaxIndex;
  }

  public String getLogMaxFileSize() {
    return logMaxFileSize;
  }

  public void setLogMaxFileSize(String logMaxFileSize) {
    this.logMaxFileSize = logMaxFileSize;
  }

  public Integer getMaxConnectionLifetimeMinutes() {
    return maxConnectionLifetimeMinutes;
  }

  public void setMaxConnectionLifetimeMinutes(Integer maxConnectionLifetimeMinutes) {
    this.maxConnectionLifetimeMinutes = maxConnectionLifetimeMinutes;
  }
  
}

//test
//private static final String user = "WEBUSER";
//private static final String password = "WEBUSER";
//private static final String system   = "mebdev.momentum.co.za";
//private static final String database = "ORBWEBTEST";
//
//prod
//private static final String user = "WEBPROD";
//private static final String password = "MEBWEB1";
//private static final String system   = "mebas400.momentum.co.za";
//private static final String database = "ORBWEBPROD";
//
