package services.as400Connector.server;

import javax.resource.ResourceException;
import javax.transaction.xa.XAException;
import javax.transaction.xa.XAResource;
import javax.transaction.xa.Xid;

import libraries.ejbUtils.logging.Logger;

public class AS400XAResource implements XAResource {
  
  private Xid                    xid;
  private AS400ManagedConnection managedCx;
  private boolean                prepared;
  private Logger                 log;
  
  public AS400XAResource(AS400ManagedConnection managedCx) {
    this.managedCx = managedCx;
    this.log       = managedCx.getLogger();
  }

  /**
   * associate the XAResource with a specific global transaction.
   * All work done from here is done for this transaction
   */
  public void start(Xid xid, int flags) throws XAException {
    log.info("Going to start new XA transaction - [START]");
    
    if (this.xid != null) {
      log.error("start transaction routine involed in improper context current xid must be null before starting new one");
      //Routine was invoked in improper context
      throw new XAException(XAException.XAER_PROTO);
    }
    
    if (flags != XAResource.TMNOFLAGS) {
      log.error("AS400 datasource only supports start with XAResource.TMNOFLAGS");
      throw new XAException("AS400 datasource only supports start with XAResource.TMNOFLAGS");
    }
    
    this.xid      = xid;
    this.prepared = false;  
  }
  
  public void end(Xid xid, int flags) throws XAException {
    
    checkXid(xid);
    
    if (flags == XAResource.TMSUSPEND) {
      log.error("AS400 datasource does not support transaction supension");
      throw new XAException("AS400 datasource does not support transaction supension");
    }
    
  }
  
  public int prepare(Xid xid) throws XAException {
    
    log.info("Going to prepare XA transaction - [PREPARE]");
    
    checkXid(xid);
    
    prepared = true;

    //TODO - check if as400 connection is still up
    //TODO - return XAResource.XA_RDONLY if we only did read only work in transaction
    
    return XAResource.XA_OK;
    
  }
  
  public void commit(Xid xid, boolean onePhase) throws XAException {
    
    log.info("Going to commit XA transaction - [COMMIT]");
    checkXid(xid);
    
    //TODO - if we only did read only work in transaction, don't try to commit
    
    try {
      managedCx.commitTransaction();
    } catch (ResourceException e) {
      log.error("Got ResourceException when trying to commit transaction",e);
      throw new XAException(e.getMessage());
    }
    
    this.xid = null;
    prepared = false;
    
  }

  public void rollback(Xid xid) throws XAException {
    
    log.info("Going to rollback XA transaction - [ROLLBACK]");
    
    checkXid(xid);

    try {
      managedCx.rollbackTransaction();
    } catch (ResourceException e) {
      log.error("Got ResourceException when trying to rollback transaction",e);
      throw new XAException(e.getMessage());
    }
    
    this.xid = null;
    prepared = false;
    
  }
  
  public void forget(Xid xid) throws XAException {
    //TODO - is this right ?
    log.info("Going to forget transaction");
    this.xid = null;
    prepared = false;
    
  }

  public boolean isSameRM(XAResource xares) throws XAException {
    return (xares == this);
  }


  public Xid[] recover(int flag) throws XAException {
    
    log.info("Going to recover transaction");
    if (prepared) {
      return new Xid[] {this.xid};
    }
    return new Xid[0];
    
  }

  
  public int getTransactionTimeout() throws XAException {
    return 0;
  }

  public boolean setTransactionTimeout(int seconds) throws XAException {
    return false;
  }
  
  private void checkXid(Xid xid) throws XAException {
    
    if (!this.xid.equals(xid)) {
      log.error("The xid provided does not match the xid of the current transaction");
      throw new XAException("The xid provided does not match the xid of the current transaction");
    }
    
  }

}
