package services.configurationConnector.server;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.resource.NotSupportedException;
import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

import libraries.configuration.ConfigurationService;
import libraries.ejbUtils.logging.Logger;

public class ConfigurationManagedConnection implements ManagedConnection {

  private ConfigurationManagedConnectionFactory mcf;
  private ConfigurationService configurationService; 
  private String module;
  private PrintWriter logWriter;
  private boolean destroyed;
  private List<ConnectionEventListener> listeners;
  private List<ConfigurationConnectionImpl> allCxHandles;
  
  public ConfigurationManagedConnection(ConfigurationManagedConnectionFactory mcf, ConfigurationService configurationService, String module) {
    this.mcf                   = mcf;
    this.configurationService  = configurationService;
    this.module                = module;
    this.destroyed             = false;
    this.listeners             = new ArrayList<ConnectionEventListener>();
    this.allCxHandles          = new ArrayList<ConfigurationConnectionImpl>();
  }

  /**
   * See association stuff - Spec 7.11
   */
  public void associateConnection(Object connection) throws ResourceException {
    checkIfDestroyed();
    ((ConfigurationConnectionImpl) connection).associateConnection(this);
  }
  
  /**
   * Reset client specific state, invalidate all handles, prepare the connection
   * to be put back in the pool Should not close the physical connection Spec 6.5.4.4
   * This is when the client has closed the handle, we want to prepare it to be used by another client
   */
  public void cleanup() throws ResourceException {
    checkIfDestroyed();
    for (ConfigurationConnectionImpl cx : allCxHandles) {
      cx.invalidate();
    }
    allCxHandles.clear();
  }
  
  /**
   * Cleanup the connection, will not be used again
   */
  public void destroy() throws ResourceException {
    if (destroyed) {
      return;
    }
    cleanup();
    //if there was a sql connection, we'd close it here, as it stands the configurationservice closes the connection after every query
    destroyed = true;
  }

  /**
   * return a new handle to a client
   */
  public Object getConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
    checkIfDestroyed();
    ConfigurationConnectionImpl cxHandle = new ConfigurationConnectionImpl(this);
    addCxHandle(cxHandle);
    return cxHandle;
  }
  
  public LocalTransaction getLocalTransaction() throws ResourceException {
    throw new NotSupportedException("Transactions not supported");
  }
  
  public XAResource getXAResource() throws ResourceException {
    throw new NotSupportedException("Transactions not supported");
  }
  
  public PrintWriter getLogWriter() throws ResourceException {
    return logWriter;
  }
  
  public void setLogWriter(PrintWriter logWriter) {
    this.logWriter = logWriter;
  }
  
  public ManagedConnectionMetaData getMetaData() throws ResourceException {
    checkIfDestroyed();
    return new ConfigurationManagedConnectionMetaData(this);
  }
  
  public void addConnectionEventListener(ConnectionEventListener listener) {
    listeners.add(listener);    
  }
  
  public void removeConnectionEventListener(ConnectionEventListener listener) {
    listeners.remove(listener);
  }
  
  void addCxHandle(ConfigurationConnectionImpl cxHandle) {
    allCxHandles.add(cxHandle);
  }
  
  void removeCxHandle(ConfigurationConnectionImpl cxHandle) {
    allCxHandles.remove(cxHandle);
  }
  
  public void closeCxHandle(ConfigurationConnectionImpl cxHandle) {
    ConnectionEvent event = new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
    event.setConnectionHandle(cxHandle);

    for (ConnectionEventListener listener : listeners) {
      listener.connectionClosed(event);
    }
  }
  
  void checkIfDestroyed() {
    if (destroyed) {
      throw new RuntimeException("This ManagedConnection has been destroyed");
    }
  }

  void notifyConnectionError() {
    ConnectionEvent event = new ConnectionEvent(this, ConnectionEvent.CONNECTION_ERROR_OCCURRED);

    for (ConnectionEventListener listener : listeners) {
      listener.connectionErrorOccurred(event);
    }
  }

  public String getModule() {
    return module;
  }
  
  public void setModule(String module) {
    this.module = module;
  }

  public ConfigurationService getConfigurationService() {
    return configurationService;
  }

  public Logger getLogger() {
    return mcf.getModuleLogger(module);
  }

}
