package services.configurationConnector.server;

import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionManager;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionFactory;
import javax.security.auth.Subject;

import libraries.configuration.ConfigurationService;
import libraries.configuration.DatabasePropertiesRetriever;
import libraries.ejbUtils.logging.Logger;
import libraries.ejbUtils.logging.LoggingService;

/**
 * This class must be a javabean - spec 17.4.2
 * @author eduardp
 *
 */
public class ConfigurationManagedConnectionFactory implements ManagedConnectionFactory {

  private static final long serialVersionUID = 1L;
  
  private PrintWriter printWriter;
  private ConfigurationConnectionManager defaultCxManager;
  private Map<String, Logger> loggers;
  private Map<String,Long> logLoadTimes;
  private ConfigurationService configurationService;
  //values for these properties are specified on the appserver admin interface 
  //or in the deployment descriptor  and injected by the container using the setters
  private String url;
  private String userName;
  private String password;
  private String driver;
  
  public ConfigurationManagedConnectionFactory() {
    defaultCxManager = new ConfigurationConnectionManager();
    loggers          = new HashMap<String, Logger>();
    logLoadTimes   = new HashMap<String, Long>();
  }
  
  //---------------------------------------------------------------------------------------------------- [ ConnectionFactory stuff ]
  /**
   * if this method is called, we need to provide a default ConnectionManager, but it will only be called if the connector is used from without the container, which is a scenario that we do not cater for
   */
  public Object createConnectionFactory() throws ResourceException {
    return new ConfigurationConnectionFactoryImpl(this,defaultCxManager);
  }

  /**
   * The container calls this method if it wants to hand a new factory obj to a client
   */
  public Object createConnectionFactory(ConnectionManager cxManager) throws ResourceException {
    return new ConfigurationConnectionFactoryImpl(this,cxManager);
  }
  //----------------------------------------------------------------------------------------------------------------------------------- 

  //---------------------------------------------------------------------------------------------------- [ ManagedConnectionFactory stuff ]
  public ManagedConnection createManagedConnection(Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
    loadConfigurationService();
    
    ConfigurationManagedConnection managedCx = new ConfigurationManagedConnection(this,configurationService,((ConfigurationConnectionRequestInfo)cxRequestInfo).getModule());
    managedCx.setLogWriter(this.getLogWriter());
    
    return managedCx;
  }

  public ManagedConnection matchManagedConnections(Set connectionSet, Subject subject, ConnectionRequestInfo cxRequestInfo) throws ResourceException {
    if ( (connectionSet == null) || (connectionSet.size() == 0) ) {
      //Could not find a suitable connection in the connection set, return null so the appserver will create a new connection
      return null;
    }
    else {
      Iterator<ConfigurationManagedConnection> itr = connectionSet.iterator();
      ConfigurationManagedConnection managedCx = itr.next();
      managedCx.setModule(((ConfigurationConnectionRequestInfo)cxRequestInfo).getModule());
      return managedCx;
    }
  }
  //----------------------------------------------------------------------------------------------------------------------------------- 

  private void loadConfigurationService() {
    if (configurationService == null) {
      configurationService = new ConfigurationService(new DatabasePropertiesRetriever(driver,url,userName,password));
    }
  }

  public Logger getModuleLogger(String module) {
    
    Properties logProps = new Properties();
    logProps.put("config.log.pattern", "%m");
    logProps.put("config.log.useRollingLog","false" );
    logProps.put("config.log.filename","/tmp/log.log" );
    logProps.put("config.log.level", "DEBUG");
    //return LoggingService.getLogger(ModuleInfo.MODULE_NAME, ModuleInfo.MODULE_NAME, logProps);
    return LoggingService.getLogger("mylogname", "config", logProps);
    
  }
  
  public PrintWriter getLogWriter() throws ResourceException {
    return printWriter;
  }

  public void setLogWriter(PrintWriter out) throws ResourceException {
    this.printWriter = out;
  }

  public String getUrl() {
    return url;
  }

  public void setUrl(String url) {
    this.url = url;
  }

  public String getUserName() {
    return userName;
  }

  public void setUserName(String userName) {
    this.userName = userName;
  }

  public String getPassword() {
    return password;
  }

  public void setPassword(String password) {
    this.password = password;
  }

  public String getDriver() {
    return driver;
  }

  public void setDriver(String driver) {
    this.driver = driver;
  }

  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((url == null) ? 0 : url.hashCode());
    result = prime * result + ((userName == null) ? 0 : userName.hashCode());
    return result;
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    final ConfigurationManagedConnectionFactory other = (ConfigurationManagedConnectionFactory) obj;
    if (url == null) {
      if (other.url != null)
        return false;
    } else if (!url.equals(other.url))
      return false;
    if (userName == null) {
      if (other.userName != null)
        return false;
    } else if (!userName.equals(other.userName))
      return false;
    return true;
  }
  
}
