package libraries.ejbUtils.logging;

import java.io.IOException;
import java.io.Serializable;
import java.util.Properties;

import org.apache.log4j.FileAppender;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.RollingFileAppender;

/**
 * Note: log4j seems to break the ejb coding restriction that no static variables are allowed, for now, we are fine with this
 * @author eduardp
 *
 */
class Log4jLogger implements libraries.ejbUtils.logging.Logger {
  
  private Logger log4jLogger;

  Log4jLogger(String logName, Properties props) {
    this(logName,"logger",props);
  }
  
  Log4jLogger(String logName, String propertyPrefix, Properties props) {

    log4jLogger = Logger.getLogger(logName);
    
    //if this logger has already been configured - reconfigure, because the properties might have changed
    if (log4jLogger.getAllAppenders().hasMoreElements()) {
      log4jLogger.removeAllAppenders();
    }
    
    String rolling = props.getProperty(propertyPrefix + ".log.useRollingLog");
    if (rolling == null) {
      throw new RuntimeException("could not find a value for the property '" + propertyPrefix + ".log.useRollingLog' for log '" + logName + "'");
    }
    boolean rollingAppender = Boolean.parseBoolean(rolling.trim().toLowerCase());

    
    int maxBackupIndex = 10;
    String maxFileSize = "1MB";
    
    if (rollingAppender) {
      String maxIndex = props.getProperty(propertyPrefix + ".log.maxIndex");
      if (maxIndex != null) {
        maxBackupIndex = Integer.parseInt(maxIndex.trim());
      }

      maxFileSize = props.getProperty(propertyPrefix + ".log.maxFileSize");
      if (maxFileSize != null) {
        maxFileSize = maxFileSize.trim();
      }
    }

    String pattern = props.getProperty(propertyPrefix + ".log.pattern");
    if (pattern == null) {
      throw new RuntimeException("could not find a value for the property '" + propertyPrefix + ".log.pattern' for log '" + logName +"'");
    }
    pattern = pattern.replaceAll("\\\\n", "\n");
    String filename = props.getProperty(propertyPrefix + ".log.filename");
    if (filename == null) {
      throw new RuntimeException("could not find a value for the property '" + propertyPrefix + ".log.filename' for log '" + logName +"'");
    }
    filename = filename.trim();

    String level = props.getProperty(propertyPrefix + ".log.level");
    if (level == null) {
      level = "debug";
    }
    level = level.trim().toLowerCase();
    Level logLevel;
    if (level.equals("info")) {
      logLevel = Level.INFO;
    }
    else if (level.equals("debug")) {
      logLevel = Level.DEBUG;
    }
    else if (level.equals("error")) {
      logLevel = Level.ERROR;
    }
    else if (level.equals("fatal")) {
      logLevel = Level.FATAL;
    }
    else if (level.equals("warn")) {
      logLevel = Level.WARN;
    }
    else if (level.equals("trace")) {
      logLevel = Level.TRACE;
    }
    else {
      logLevel = Level.OFF;
    }

    PatternLayout layout = new PatternLayout(pattern);
    FileAppender appender;
    try {
      if (rollingAppender) {
        appender = new RollingFileAppender(layout,filename,true);
        ((RollingFileAppender)appender).setMaxBackupIndex(maxBackupIndex);
        ((RollingFileAppender)appender).setMaxFileSize(maxFileSize);
      }
      else {
        appender = new FileAppender(layout,filename,true);
      }
    } catch (IOException e) {
      throw new RuntimeException("Failed to create file appender",e); 
    }
    appender.setBufferedIO(false);
    appender.activateOptions();
    log4jLogger.addAppender(appender);
    log4jLogger.setLevel(logLevel);

  }

  public void debug(Object message) {
    log4jLogger.debug(message);
  }

  public void debug(Object message, Throwable t) {
    log4jLogger.debug(message, t);
  }

  public void error(Object message) {
    log4jLogger.error(message);
  }

  public void error(Object message, Throwable t) {
    log4jLogger.error(message, t);
  }

  public void fatal(Object message) {
    log4jLogger.fatal(message);
  }

  public void fatal(Object message, Throwable t) {
    log4jLogger.fatal(message, t);
  }

  public void info(Object message) {
    log4jLogger.info(message);
  }

  public void info(Object message, Throwable t) {
    log4jLogger.info(message, t);
  }

  public void warn(Object message) {
    log4jLogger.warn(message);
  }

  public void warn(Object message, Throwable t) {
    log4jLogger.warn(message, t);
  }

  public libraries.ejbUtils.logging.Level getLevel() {
    if (log4jLogger.getLevel().equals(Level.FATAL)) {
      return libraries.ejbUtils.logging.Level.FATAL;
    }
    else if (log4jLogger.getLevel().equals(Level.INFO)) {
      return libraries.ejbUtils.logging.Level.INFO;
    }
    else if (log4jLogger.getLevel().equals(Level.ERROR)) {
      return libraries.ejbUtils.logging.Level.ERROR;
    }
    else if (log4jLogger.getLevel().equals(Level.WARN)) {
      return libraries.ejbUtils.logging.Level.WARN;
    }
    else {
      return libraries.ejbUtils.logging.Level.DEBUG;
    }
  }

  public boolean isDebugEnabled() {
    return log4jLogger.isDebugEnabled();
  }

  public boolean isInfoEnabled() {
    return isInfoEnabled();
  }
  
  public boolean isEnabledFor(libraries.ejbUtils.logging.Level level) {
    switch(level) {
    case INFO:
      return log4jLogger.isEnabledFor(Level.INFO);
    case WARN:
      return log4jLogger.isEnabledFor(Level.WARN);
    case ERROR:
      return log4jLogger.isEnabledFor(Level.ERROR);
    case FATAL:
      return log4jLogger.isEnabledFor(Level.FATAL);
    default:
      return log4jLogger.isEnabledFor(Level.DEBUG);
    }
  }

}
